--------------------------------------------------------------------------------
-- copyright notice:
--------------------------------------------------------------------------------
-- copyright (c) 2007 by
--
-- CESYS GmbH / GERMANY
--
-- http://www.cesys.com
--------------------------------------------------------------------------------
-- author:
--------------------------------------------------------------------------------
-- Stefan Fischer <stefan.fischer@cesys.com>
--------------------------------------------------------------------------------
-- license:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS NOT FREE! IT IS FOR USE TOGETHER WITH THE CESYS EFM01
-- USB CARD (ARTICLE-NR.: C1050-4107) ONLY! YOU ARE NOT ALLOWED TO MODIFY AND
-- DISTRIBUTE OR USE IT WITH ANY OTHER HARDWARE, SOFTWARE OR ANY OTHER KIND OF
-- ASIC OR PROGRAMMABLE LOGIC DESIGN WITHOUT THE EXPLICIT PERMISSION OF THE
-- COPYRIGHT HOLDER!
--------------------------------------------------------------------------------
-- disclaimer of warranty:
--------------------------------------------------------------------------------
-- THIS SOURCECODE IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT
-- THERE IS NO WARRANTY OR SUPPORT FOR THIS SOURCECODE. THE COPYRIGHT HOLDER 
-- PROVIDES THIS SOURCECODE "AS IS" WITHOUT WARRANTY OF ANY KIND,
-- EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
-- WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
-- THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THIS SOURCECODE IS WITH
-- YOU. SHOULD THIS SOURCECODE PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL 
-- NECESSARY SERVICING, REPAIR OR CORRECTION.
--
-- IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES, INCLUDING
-- ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
-- USE OR INABILITY TO USE THIS SOURCECODE (INCLUDING BUT NOT LIMITED TO LOSS OF
-- DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD 
-- PARTIES OR A FAILURE OF THIS SOURCECODE TO OPERATE WITH ANY OTHER
-- SOFTWARE-PROGRAMS, HARDWARE-CIRCUITS OR ANY OTHER KIND OF ASIC OR
-- PROGRAMMABLE LOGIC DESIGN), EVEN IF THE COPYRIGHT HOLDER HAS BEEN ADVISED OF 
-- THE POSSIBILITY OF SUCH DAMAGES.
--------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.all;

use work.wishbone.all;


entity wb_sl_flash is
  generic
  (
    nr_of_irqoports : positive := 1;
    nr_of_irqiports : positive := 1;
    nr_of_dbgports : positive := 1
  );
  port
  (
    SYSCON_I : in rec_syscon_port;
    MASTER_I : in rec_master_port;
    SLAVE_O : out rec_slave_port;

    IRQ_I : in std_logic_vector((nr_of_irqiports-1) downto 0);
    IRQ_O : out std_logic_vector((nr_of_irqoports-1) downto 0);

    flash_s_n_o : out std_logic;
    flash_c_o : out std_logic;
    flash_hold_n_o : out std_logic;
    flash_w_n_o : out std_logic;
    flash_d_o : out std_logic;
    flash_q_i : in std_logic;

    debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
  );
end wb_sl_flash;


architecture RTL of wb_sl_flash is

  signal wishbone : rec_wishbone_signal := wishbone_default;

  signal irqo : std_logic_vector((nr_of_irqoports-1) downto 0) := (others => '0');
  signal irqi : std_logic_vector((nr_of_irqiports-1) downto 0) := (others => '0');

  signal flash_s_n : std_logic := '1';
  signal flash_c : std_logic := '0';
  signal flash_hold_n : std_logic := '1';
  signal flash_w_n : std_logic := '1';
  signal flash_d : std_logic := '0';
  signal flash_q : std_logic := '0';

  component flash_ctrl is
    generic
    (
      nr_of_dbgports : integer := 1
    );
    port
    (
      rst : in std_logic;
      clk : in std_logic;
      
      usr_access_i : in std_logic;
      usr_erase_i : in std_logic;
      usr_we_i : in std_logic;
      usr_adr_i : in std_logic_vector(16 downto 0);
      usr_dati_i : in std_logic_vector(31 downto 0);
      usr_dato_o : out std_logic_vector(31 downto 0);
      usr_ok_o : out std_logic;
  
      flash_s_n_o : out std_logic;
      flash_c_o : out std_logic;
      flash_hold_n_o : out std_logic;
      flash_w_n_o : out std_logic;
      flash_d_o : out std_logic;
      flash_q_i : in std_logic;
  
      debug : inout std_logic_vector((nr_of_dbgports-1) downto 0)
    );
  end component;

  signal rst : std_logic := '1';
  signal clk : std_logic := '0';

  signal usr_access : std_logic := '0';
  signal usr_erase : std_logic := '0';
  signal usr_we : std_logic := '0';
  signal usr_adr : std_logic_vector(16 downto 0) := (others  => '0');
  signal usr_dati : std_logic_vector(31 downto 0) := (others  => '0');
  signal usr_dato : std_logic_vector(31 downto 0) := (others  => '0');
  signal usr_ok : std_logic := '0';

begin

  -- connect ports to internal signals
  ------------------------------------
  
  -- ports of WISHBONE bus
  wishbone.syscon <= SYSCON_I;
  wishbone.master <= MASTER_I;
  SLAVE_O <= wishbone.slave;
  
  -- ports of interrupt bus
  irqi <= IRQ_I;
  IRQ_O <= irqo;

  -- ports of SPI FLASH
  flash_s_n_o <= flash_s_n;
  flash_c_o <= flash_c;
  flash_hold_n_o <= flash_hold_n;
  flash_w_n_o <= flash_w_n;
  flash_d_o <= flash_d;
  flash_q <= flash_q_i;

  debug <=
  (
    others => 'Z'
  );
  
  -- instantiate low level flash controller
  -----------------------------------------

  inst_flash_ctrl : flash_ctrl
  generic map
  (
    nr_of_dbgports => 1
  )
  port map
  (
    rst => rst,
    clk => clk,
  
    usr_access_i => usr_access,
    usr_erase_i => usr_erase,
    usr_we_i => usr_we,
    usr_adr_i => usr_adr,
    usr_dati_i => usr_dati,
    usr_dato_o => usr_dato,
    usr_ok_o => usr_ok,

    flash_s_n_o => flash_s_n,
    flash_c_o => flash_c,
    flash_hold_n_o => flash_hold_n,
    flash_w_n_o => flash_w_n,
    flash_d_o => flash_d,
    flash_q_i => flash_q,

    debug => open
  );

  rst <= wishbone.syscon.rst;
  clk <= wishbone.syscon.clk;

  usr_access <= wishbone.master.cyc and wishbone.master.stb;
  -- applying flash erase command with wishbone write address decoding
  -- decoding only the msb for saving FPGA resources
  -- command register address is one position over flash memory space
  usr_erase <= wishbone.master.cyc and wishbone.master.stb and wishbone.master.we and wishbone.master.adr(rng_flash_adr'high+1);
  usr_we <= wishbone.master.we;
  usr_adr <= wishbone.master.adr(rng_flash_adr);
  usr_dati <= wishbone.master.dat;
  wishbone.slave.dat <= usr_dato;
  wishbone.slave.ack <= usr_ok;

end RTL;